#include <vector>
#include <memory>
#include <limits>
#include <glpk.h>


typedef std::vector<std::vector<double>> Matrix ;
typedef std::vector<double> RowVector ;

class GlpkInterface {
public:
  GlpkInterface() {
    lp_ = glp_create_prob() ;
  }

  void AddRows(const Matrix& mat) {
    unsigned cons_num = mat.size() ;
    unsigned vari_num = mat[0].size() - 1 ;
    if (glp_get_num_rows(lp_) == 0) {
      glp_add_cols(lp_, vari_num) ;
      for (unsigned j = 0; j < vari_num; ++ j) {
        glp_set_col_bnds(lp_, j+1, GLP_FR, 0.0, 0.0) ;
      } 
    }
    int row_idx = glp_add_rows(lp_, cons_num) ;
    std::unique_ptr<int[]> idx_array(new int[vari_num+1]) ;
    std::unique_ptr<double[]> val_array(new double[vari_num+1]) ;
    int* idx_ptr = idx_array.get() ;
    double* val_ptr = val_array.get() ;
    for (unsigned j = 1; j < vari_num+1; ++ j) {
      idx_ptr[j] = j ; 
    }
    for (unsigned i = 0; i < cons_num; ++ i) {
      for (unsigned j = 0; j < vari_num; ++ j) {
        val_ptr[j+1] = mat[i][j] ;
      }
      glp_set_mat_row(lp_, row_idx+i, vari_num, idx_ptr, val_ptr);
      glp_set_row_bnds(lp_, row_idx+i, GLP_UP, 0.0, mat[i][vari_num]) ;
    } 
    for (unsigned i = 0; i < mat.size(); ++ i) {
      cons_.push_back(mat[i]) ;
    }
  }

  void SetRow(unsigned idx, const RowVector& vec) {
    unsigned vari_num = vec.size() - 1 ;
    std::unique_ptr<int[]> idx_array(new int[vari_num+1]) ;
    std::unique_ptr<double[]> val_array(new double[vari_num+1]) ;
    int* idx_ptr = idx_array.get() ;
    double* val_ptr = val_array.get() ;
    for (unsigned j = 1; j < vari_num+1; ++ j) {
      idx_ptr[j] = j ; 
      val_ptr[j] = vec[j-1] ;
    }
    glp_set_mat_row(lp_, idx+1, vari_num, idx_ptr, val_ptr);
    glp_set_row_bnds(lp_, idx+1, GLP_UP, 0.0, vec[vari_num]) ;
    cons_[idx] = vec ;
  }

  void SwapRows(unsigned idx1, unsigned idx2) {
    if (idx1 == idx2) return ;
    RowVector tmp_row = cons_[idx1] ;
    SetRow(idx1, cons_[idx2]) ;
    SetRow(idx2, tmp_row) ;
  }

  void DeleteRowBack() {
    int idx[] = {0, (int)cons_.size()} ;
    glp_del_rows(lp_, 1, idx) ;
    cons_.pop_back() ;
  }

  void Simplex() {
    glp_simplex(lp_, NULL) ;
  }
  
  void SetObj(const RowVector& obj) {
    for (unsigned i = 0; i < obj.size(); ++ i) {
      glp_set_obj_coef(lp_, i+1, obj[i]) ;
    }
  }

private: 
  glp_prob* lp_ ;
  Matrix cons_ ;
} ;
